c**********************************************************************
c               
c       This double precision function evaluates the integrand of 
c       the orientational integral in the definition of the starting 
c       vector.  The integral over the gamma Euler angle is done 
c       analytically.
c
c       Notes:
c               1) l,m quantum numbers are passed via a 
c                  common block.
c               2) For more information on the evaluation of the 
c                  modified Bessel and associated Legendre functions
c                  see the respective routines.
c
c       written by AP 17-FEB-94, based on code written by DJS 10-SEP-87
c
c       Includes:
c               eprdat.inc
c
c       Uses:
c               bessi.f
c               plgndr.f
c
c**********************************************************************
c
      double precision function fzs(z)
c
      double precision z
c
      include 'eprprm.inc'
c
      integer iff,lr,kr,iberr
      common/ifdat/iff,lr,kr,iberr
c
      double precision a1,a2,b,c1,c2
	integer ierr
c
      double precision dsq24,dsq360
      parameter (dsq24=4.89897948556635619639,
     #     dsq360=18.97366596101027599199)
c
      double precision bessi,plgndr
      external bessi,plgndr
c
c######################################################################
c
      iberr=0
c 
      if (iff.eq.0) then
                                ! cage mean field factor
         b=0.5d0*b10*plgndr(1,0,z)+0.5d0*b20*plgndr(2,0,z)
         b=dexp(b)
         fzs=b*plgndr(lr,0,z)
      else if (iff.eq.1) then
                                ! interaction factor
         c1=0.5d0*(c20*plgndr(2,0,z)+c40*plgndr(4,0,z))
         c1=dexp(c1)
         c2=c22*plgndr(2,2,z)/dsq24+c42*plgndr(4,2,z)/dsq360
         c2=bessi(kr/2,c2,ierr)
         fzs=c1*c2*plgndr(lr,kr,z)
	   if (ierr.ne.0) iberr=-1
      else if (iff.eq.2) then
                                ! probe mean field factor
         a1=0.5d0*(a20*plgndr(2,0,z)+a40*plgndr(4,0,z))
         a1=dexp(a1)
         a2=a22*plgndr(2,2,z)/dsq24+a42*plgndr(4,2,z)/dsq360
         a2=bessi(kr/2,a2,ierr)
         fzs=a1*a2*plgndr(lr,kr,z)
	   if (ierr.ne.0) iberr=-1
      end if
c
c----------------------------------------------------------------------
c     return to caller
c----------------------------------------------------------------------
c
      return
      end



