c**********************************************************************
c                        =========================
c                           subroutine MATRSR
c                        =========================
c
c       Subroutine for calculating matrix elements for EPRSR calculation.
c       Based on MATRLL, written by AP 17-FEB-94 for EPRSR calculation.
c
c       Includes :
c               stddim.inc
c               rndoff.inc
c               eprdat.inc
c               eprmat.inc
c               maxl.inc
c               srsdat.inc
c               srsflg.inc
c               srsind.inc
c               srssto.inc
c
c       Uses:
c               caldlm.f
c               ipar.f
c               w3j.f
c
c**********************************************************************
c
      subroutine matrll(ierr)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'rndoff.inc'
      include 'eprmat.inc'
      include 'eprprm.inc'
      include 'srsset.inc'
      include 'maxl.inc'
      include 'srsdat.inc'
      include 'srsflg.inc'
      include 'srsind.inc'
      include 'srssto.inc'
c
      integer ierr
      integer i,j,is,it,isa,isb,ikr,ikc,ih,imr,imc
      integer ipp,jin,jfi,iskr,iskc,ismr,ismc,iir,iic,isir,isic
      integer neli,nel,nelr
      integer nrow,ncol
c
      double precision zero,cliou,cgam,cgamf,factr,factc
      double precision c,fact,d2
c
      double precision ct,be,plk
      parameter (be=9.2731d-21,plk=1.05443d-27)
c
      logical check,fdiff,fliou,delta
c
      parameter (zero=0.d0,d2=0.707106781d0)
c
      double precision w3j
      integer ipar 
c
c######################################################################
c
      if (ndim.gt.mxdim) then
        ierr=2
        go to 9999
      end if
c
c----------------------------------------------------------------------
c     parameters
c----------------------------------------------------------------------
c
c.... wigner matrices for psi and bed .................................
c
      call caldlm(d2psi,psi)
      call caldlm(d2bed,bed)
c      write (*,*) 'w-expansion done'
c
c.... high field approximation ........................................
c
      hfdflg=.false.
c
c.... rescaled diffusion constants ....................................
c
      ct=g0*be/plk
      drt=0.5d0*(dx+dy)/ct
      dpt=dz/ct
      d0rt=0.5d0*(d0x+d0y)/ct
      d0pt=d0z/ct
c
c----------------------------------------------------------------------
c     Define or read reduced matrix elements and potential parameters
c----------------------------------------------------------------------
c
c.... rearranging potential parameter in ordered array [see INTFC] ....
c
      call intfc
c      write (*,*) 'u-expansion done'
c
c.... expansion of the F function [see CFPOT] .........................
c
      call cfpot
c      write (*,*) 'f-expansion done'
c
c.... reduced matrix elements for the Lioville operator [see LSTORE] ..
c
      call lstore
c     write (*,*) 'l-expansion done'
c
c----------------------------------------------------------------------
c     Initialize counters
c----------------------------------------------------------------------
c
      nelr=0
      neli=0
      nel=0
c
c----------------------------------------------------------------------
c     **** Loop over rows ***
c----------------------------------------------------------------------
c
      do 400 nrow=1,ndim
        lr=bss0(1,nrow)
        mr=bss0(2,nrow)
        kr=bss0(3,nrow)
        jr=bss0(4,nrow)
        nr=bss0(5,nrow)
        ir=bss0(6,nrow)
        ipnr=bss0(7,nrow)
        imr=bss0(8,nrow)
        iqnr=bss0(9,nrow)
        ipsr=bss0(10,nrow)
        iqsr=bss0(11,nrow)
c
        jzmat(nrow)=neli+1
        kzmat(nrow)=nelr+1
c
c----------------------------------------------------------------------
c   **** Loop over columns (note only upper diagonal) ****
c----------------------------------------------------------------------
c
        do 300 ncol=nrow,ndim
          lc=bss0(1,ncol)
          mc=bss0(2,ncol)
          kc=bss0(3,ncol)
          jc=bss0(4,ncol)
          nc=bss0(5,ncol)
          ic=bss0(6,ncol)
          ipnc=bss0(7,ncol)
          imc=bss0(8,ncol)
          iqnc=bss0(9,ncol)
          ipsc=bss0(10,ncol)
          iqsc=bss0(11,ncol)
c
c.... find normalization factors of the symmetrized elements ..........
c
          if (kr.eq.0) then
             factr=0.5d0
             ikr=ipar(lr)
          else
             factr=d2
             ikr=1
          end if
c
          if (ir.eq.0) then
             factr=0.5d0*factr
             iir=ipar(jr)
          else
             factr=d2*factr
             iir=1
          end if
c
          if (mr.eq.0.and.nr.eq.0.and.ipnr.eq.0) then
             factr=factr*0.5d0
          else
             factr=factr*d2
          end if
c
          if (kc.eq.0) then
             factc=0.5d0
             ikc=ipar(lc)
          else
             factc=d2
             ikc=1
          end if
c
          if (ic.eq.0) then
             factc=0.5d0*factc
             iic=ipar(jc)
          else
             factc=d2*factc
             iic=1
          end if
c
          if (mc.eq.0.and.nc.eq.0.and.ipnc.eq.0) then
             factc=factc*0.5d0
          else
             factc=factc*d2
          end if
c
          fact=factr*factc
c
c.... find parities of the symmetrized elements .......................
c
          iskr=ipar(lr+kr)
          isir=ipar(jr+ir)
          ismr=ipar(lr+jr+mr+nr)
          iskc=ipar(lc+kc)
          isic=ipar(jc+ic)
          ismc=ipar(lc+jc+mc+nc)
c
c----------------------------------------------------------------------
c     Diffusion operator
c----------------------------------------------------------------------
c
          cgam=zero
c
c......................................................................
c.......... calculate cgam isotropic ..................................
c......................................................................
c
          check=lr.eq.lc.and.mr.eq.mc.and.kr.eq.kc.and.
     #    jr.eq.jc.and.nr.eq.nc.and.ir.eq.ic.and.ipnr.eq.ipnc.and.
     #    imr.eq.imc.and.iqnr.eq.iqnc.and.ipsr.eq.ipsc.and.
     #    iqsr.eq.iqsc.and.ikr.eq.ikc.and.iir.eq.iic
c
          if (check) then
             cgam=drt*lr*(lr+1)+(dpt-drt)*kr*kr+
     #            d0rt*jr*(jr+1)+(d0pt-d0rt)*ir*ir
          end if
c
c......................................................................
c.......... calculate cgam function ...................................
c......................................................................
c
          fdiff=iabs(ipnr).eq.iabs(ipnc).and.iqnr.eq.iqnc.and.
     #    ipsr.eq.ipsc.and.iqsr.eq.iqsc.and.
     #    iabs(lr-lc).le.mxpl.and.iabs(jr-jc).le.mxpl.and.
     #    ikr.eq.ikc.and.imr.eq.imc.and.iir.eq.iic
c
          if (fdiff) then
c
            it=(1+ikr*ikc)*(1+imr*imc)*(1+iir*iic)
c
            cgamf=zero
c
c.......... exclude non-zero matrix elements due to pI ................
c
            ipp=ipnr*ipnc
            if (ipp.eq.0) then
               jin=1
               jfi=8
            else if (ipp.gt.0) then
               jin=1
               jfi=4
            else
               jin=5
               jfi=8
            end if
c
c.......... matrix elements (symmetrized basis) ......................
c
            do j=jin,jfi
              if (j.eq.1) then
                is=1
              else if (j.eq.2) then
                is=ikc*iskc
              else if (j.eq.3) then
                is=iic*isic
              else if (j.eq.4) then
                is=iic*isic*ikc*iskc
              else if (j.eq.5) then
                is=imc*ismc
              else if (j.eq.6) then
                is=ikc*iskc*imc*ismc
              else if (j.eq.7) then
                is=iic*isic*imc*ismc
              else if (j.eq.8) then
                is=iic*isic*ikc*iskc*imc*ismc
              end if
c
              call isym(1,j)
c
c.......... matrix elements (unsymmetrized basis) ....................
c
              call gamma(c)
c             write(*,*) 'gamma done'
c
              cgamf=cgamf+is*c
            end do
c
            cgamf=fact*it*cgamf
            cgam=cgam+cgamf
c
          end if
c
c----------------------------------------------------------------------
c     Liouville operator
c
c     NOTE: see Appendix B p.3936-3937 in M,I,I,M & F for
c           details.
c----------------------------------------------------------------------
c
          cliou=zero
c
          fliou=jr.eq.jc.and.iabs(nr).eq.iabs(nc).and.ir.eq.ic
     #    .and.(delta(lr,2,lc).or.lr.eq.lc)
     #    .and.ikr.eq.ikc
c
          if (.not.hfdflg) then
             fliou=fliou.and.imr.eq.imc
          end if
c
c......................................................................
c.......... calculate cliou if not zero ...............................
c......................................................................
c
          if (fliou) then
c
c.......... apply m-symmetrization in the HF approx ...................
c
            if (hfdflg) then
              it=(1+ikr*ikc)*(1+iir*iic)
              ih=2
            else
              it=(1+ikr*ikc)*(1+imr*imc)*(1+iir*iic)
              ih=1
            end if
c
c.......... matrix elements (symmetrized basis) ......................
c
            do i=1,ih
              if (i.eq.1) then
                isa=1
              else
                isa=imr*ismr
              end if
c
              do j=1,8
               if (j.eq.1) then
                 isb=isa
               else if (j.eq.2) then
                 isb=isa*ikc*iskc
               else if (j.eq.3) then
                 isb=isa*iic*isic
               else if (j.eq.4) then
                 isb=isa*iic*isic*ikc*iskc
               else if (j.eq.5) then
                 isb=isa*imc*ismc
               else if (j.eq.6) then
                 isb=isa*ikc*iskc*imc*ismc
               else if (j.eq.7) then
                 isb=isa*iic*isic*imc*ismc
               else if (j.eq.8) then
                 isb=isa*iic*isic*ikc*iskc*imc*ismc
               end if
               call isym(i,j)
c
c.......... matrix elements (unsymmetrized basis) ....................
c
               call liouv(c)
               cliou=cliou+isb*c
              end do
            end do
c
            cliou=fact*it*cliou
c
          end if
c
c----------------------------------------------------------------------
c             Store matrix element
c----------------------------------------------------------------------
c
c             Diagonal elements
c
              if (nrow.eq.ncol) then
                  zdiag(1,nrow)=cgam
                  zdiag(2,nrow)=-cliou
                else
c
c              Off-diagonal elements
c
                  if (abs(cliou).gt.rndoff) then
                    nel=nel+1
                    if (nel.gt.mxel) then
                      ierr=1
                      go to 9999
                    else
                      neli=neli+1
                      zmat(neli)=-cliou
                      izmat(neli)=ncol
                    end if
                  end if
c
                  if (abs(cgam).gt.rndoff) then
                    nel=nel+1
c        write(*,1000) nel,lr,mr,kr,jr,nr,ir,lc,mc,kc,jc,nc,ic,cgam
c1000 format(2x,i5,',',2x,12(i2,','),f12.6)
                    if (nel.gt.mxel) then
                      ierr=1
                      go to 9999
                    else
                      nelr=nelr+1
                      zmat(mxel-nelr+1)=cgam
                      izmat(mxel-nelr+1)=ncol
                    end if
                  end if
                end if
c
c----------------------------------------------------------------------
c       end loop over columns
c----------------------------------------------------------------------
c
 300            continue
c
c----------------------------------------------------------------------
c     end loop over rows
c----------------------------------------------------------------------
c
 400  continue
c
c----------------------------------------------------------------------
c     return to calling program
c----------------------------------------------------------------------
c
      ierr=0
      neltot=nel
      nelre=nelr
      nelim=neli
c
      jzmat(ndim+1)=neli+1
      kzmat(ndim+1)=nelr+1
c
 9999 return
c
 9000 format (//,'Storing reduced matrix elements and other ',
     #'parameters ... ',$)
 9010 format ('done.')
c
      end











