c Based on EPRFSL - Version 1.4     5/2/93
c--------------------------------------------------------------------------
c                         ====================
c                          subroutine EPRFSS
c                         ====================
c
c   Field-swept conjugate gradients calculation used to determine a
c   minimum truncation set for an EPRSR slow-motional spectrum.
c   The main part of this routine consists of code from the EPRBL
c   program in versions 1.0 through 1.3 of the EPRLL program family.
c
c   The routine uses the conjugate gradients method to solve the matrix
c   equation x = A*b, where the diagonal of A depends on the field for
c   which the spectrum is being calculated. The problem is solved for
c   the specified field values across the spectrum, and at each point
c   the overlap of each basis vector with the solution x is calculated.
c   The maximum overlap of each vector with the solution is returned
c   and the field value at which the maximum occurred are returned
c   in the basis array.
c  
c   Upon entry to subroutine EPRFSS, it is assumed that:
c      (1) the basis set indices in /lindex/ have been initialized
c      (2) the stochastic Liouville matrix has been calculated in zmat
c      (3) the starting vector has been calculated and is passed in the
c          b vector
c      (4) if the "report" flag is true, the log file has already been 
c          opened on unit lulog
c
c     Written by DEB using program EPRBL from version 1.3
c     Renamed by AP 17-FEB-94, for compatability with EPRSR package.
c--------------------------------------------------------------------------
c
      subroutine eprfss( b,basis,spect,bmax,terrmx,nstpmx,cpufs,ntotal,
     #                   report )
      implicit none
      include 'stddim.inc'
c
      double precision b(2,mxdim),basis(2,mxdim),spect(2,mxpt),bmax,
     #     terrmx
      integer nstpmx,ntotal
      logical report
c
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'pidef.inc'
      include 'fnames.inc'
      include 'indexl.inc'
      include 'timer.inc'  
c
      double precision zero,one
      parameter (zero=0.0D0,one=1.0D0)
c
      character flgstr*5
      logical fexist,pcflag
c
      integer i,j,n,ncalc,ierr,ndone
c
      real cpufs
      double precision x(2,mxdim),y(2,mxdim),invrs(mxdim)
c
      double precision test,scale,truerr,error,field,
     #                 dfield,bshft,bcut,cspsi,cputmp
c
      integer maxle,maxlo,maxk,maxm,maxpi,minle,minlo,
     #     mink,minm,minpi,ipar,nbas,icut,crdim,indim,iort,
     #     picr,qicr,lcr,kcr,mcr,maxi,mini
c
      complex*16 temp,zdotu
      external zdotu,ipar
c
c#####################################################################
c.............................
c     UNIX-specific timing call
c
c     cpu3 = dtime( tarray )
c..............................
c
      nstpmx=0
      ntotal=0
      bmax=zero
      terrmx=zero
      cpufs=0.0
c
c----------------------------------------------------------------------
c     Zero basis projection array
c----------------------------------------------------------------------
c
        do 110 i=1,mxdim
          basis(1,i)=zero
          basis(2,i)=zero
 110    continue
c
c----------------------------------------------------------------------
c     Calculate the inverse of the preconditioning matrix
c----------------------------------------------------------------------
c
        do 130 j=1,ndim
          invrs(j)=one/sqrt(zdiag(1,j)+shiftr)
 130    continue
c
        if (report) write (lulog,3000)
c
c======================================================================
c     loop over field positions
c======================================================================
c
        dfield=(fieldf-fieldi)/dble(nfield-1)
c
        do 200 i=1,nfield
c
        if (i.eq.1.or.mod(i,10).eq.0.or.i.eq.nfield)
     #      write (*,*) 'field step number= ',i
c
c----------------------------------------------------------------------
c     do the preconditioned CG iteration
c----------------------------------------------------------------------
c
          do 210 j=1,ndim
	    x(1,j)=zero
	    x(2,j)=zero
  210     continue
c
          field=fieldi+(i-1)*dfield
          temp=dcmplx(shiftr,shifti-field)
c
          pcflag=.false.
          call cspccg( b,ndim,nstep,cgtol,temp,invrs,x,ndone,
     #                 error,pcflag )
c
c         cpu3 = dtime( tarray )
          cpufs = cpufs+cpu3
c
c---------------------------------------------------------------------
c     find the true error in the solution
c----------------------------------------------------------------------
c
          call scmvm(x,y,ndim)
          call zaxpy(x,y,temp,ndim)
c
	  truerr=zero
          do 220 j=1,ndim
            y(1,j)=b(1,j)-y(1,j)
            truerr=truerr+y(1,j)*y(1,j)  
            y(2,j)=b(2,j)-y(2,j)
	    truerr=truerr+y(2,j)*y(2,j)
 220      continue
c
          truerr=sqrt(truerr)
c
	  if (truerr.gt.terrmx) terrmx=truerr
          ntotal=ntotal+abs(ndone)+1
c
c----------------------------------------------------------------------
c     write out messages if required
c----------------------------------------------------------------------
c
          if ((ndone.gt.0).and.report) write(lulog,3010) i,ndone,error,
     #         truerr,cpu3
c
          if (ndone.le.0) then
            write(luttyo,3020) i,nstep,-ndone,cgtol,error,truerr,cpu3
            if (report) write(lulog,3020) i,nstep,-ndone,cgtol,error,
     #           truerr,cpu3
          end if
          if (iabs(ndone).gt.nstpmx) nstpmx=iabs(ndone)
c
c----------------------------------------------------------------------
c     store spectral information
c----------------------------------------------------------------------
c
          temp=zdotu(b,x,ndim)
          spect(1,i)=spect(1,i)+dreal(temp)/pi
          spect(2,i)=spect(2,i)+dimag(temp)/pi
c
c----------------------------------------------------------------------
c     sift through the solution vector
c----------------------------------------------------------------------
c
          scale=1.0D0/abs(temp)
          do 230 j=1,ndim
            test=sqrt(x(1,j)*x(1,j)+x(2,j)*x(2,j))*scale
            if (test.gt.basis(2,j)) then
              basis(2,j)=test
              basis(1,j)=field
            end if
            if (basis(2,j).gt.bmax) bmax=basis(2,j)
 230      continue
c
c======================================================================
c     end of loop over field positions
c======================================================================
c
 200    continue
c
        if (report) then 
           write(lulog,3030)
           write(lulog,3100) ntotal,terrmx
           write (lulog,3110) cpufs
c
           write(luttyo,3100) ntotal,terrmx
           write (luttyo,3110) cpufs
        end if
c
        return
c
c  ##### Format statements ############################################
c
 3000 format(/,2x,'+--------+-----------+----------------+',
     #            '----------------+',
     #       /,2x,'| Step   | Number of |   Modulus of   |',
     #            '  L2 norm of    |',
     #       /,2x,'| number | CG steps  |   rectanorm    |',
     #            '  true residual |',
     #       /,2x,'|--------+-----------+----------------+',
     #            '----------------|')
 3010 format(  2x,'| ',i6,' |   ',i6,'  | ',g14.7,' | ',g14.7,' | ',
     #       f10.2)
 3020 format(//,10x,'*** CG calculation ',i3,' did not converge ***',
     #     /,10x,'*** Maximum number of CG steps  : ',i4,' ***',
     #     /,10x,'*** Number of CG steps executed : ',i4,' ***',
     #     /,10x,'*** Maximum CG error allowed    : ',g14.7,' ***',
     #     /,10x,'*** Final CG error              : ',g14.7,' ***',
     #     /,10x,'*** True error                  : ',g14.7,' ***',
     #     /,10x,'*** CPU time (seconds)          : ',g14.2,' ***')
 3030 format(  2x,'+--------+-----------+----------------+',
     #            '----------------+',/)
c
 3100 format(/,2x,'Total number of matrix-vector multiplies : ',i5,
     #       /,2x,'Maximum true error : ',g14.7)
 3110 format(2x,'Total CPU time for swept-field calc (seconds) : ',
     #     f10.2)
        end

