c       Version 1.4     4/13/93
c**********************************************************************
c
c                       =================
c                       PROGRAM : PRUNEL
c                       =================
c
c       This program reads in the basis set truncation file <name>.bss 
c       written by EPRBL, and creates an output file containing a
c       list of basis vector indices determined using a prespecified
c       cutoff tolerance.
c
c       Includes:
c               stdio.inc
c               eprdat.inc
c               stddim.inc
c               fnames.inc
c               indexl.inc
c               rndoff.inc
c
c       Uses:
c               ipar.f
c               getids.f
c               setnam.f
c               setflg.f
c               lbasix.f
c               rddat.f
c
c**********************************************************************
c
      program prunel
c
      implicit none
c
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'stddim.inc'
      include 'fnames.inc'
      include 'indexl.inc'
      include 'rndoff.inc'
c
      integer i,j,ierr,ifile,nbss,nfiles
      integer maxle,minle,maxlo,minlo,maxm,minm,maxk,mink,maxj,minj,
     #        maxn,minn,maxpi,minpi,maxi,mini
      integer nbas,nmts
      double precision zero
      parameter (zero=0.0D0)

      double precision basis(2,mxdim),bmax
      logical fexist
c
c######################################################################
c
      write(luttyo,1000)
      write(luttyo,1010)
c
      ifile=0
      call getids(nfiles,mxcalc)
      if (nfiles.eq.0) goto 9999
c
 1    ifile=ifile+1
      if (ifile.gt.nfiles) goto 9999
      fileid=files(ifile)
      call setnam
      call setflg( flags(i) )
c
c----------------------------------------------------------------------
c     read input parameters from input files
c----------------------------------------------------------------------
c
        inquire(file=prname,exist=fexist)
c
        if (fexist) then
          call rddat(prname,namlth,ierr)
          if (ierr.eq.-100) then
            write (luttyo,1201) prname(:namlth)
            go to 1
          else if (ierr.ne.0) then
            write (luttyo,1200) prname(:namlth)
            go to 1
          end if
        else
          write (luttyo,1100) prname(:namlth)
          go to 1
        endif
c
        write (luttyo,5000) ifile,bsname(:namlth)
c
      open (unit=ludisk,file=bsname(:namlth),status='old',
     #     access='sequential',form='unformatted')
      read (ludisk) ndim,lemx,lomx,mmx,kmx,jmx,nmx,imx,ipnmx
c     read (ludisk) (basis(1,i),basis(2,i),i=1,ndim)
      read (ludisk) (i,l1(i),m1(i),k1(i),j1(i),n1(i),i1(i),
     #                      basis(1,i),basis(2,i),i=1,ndim)
      close (unit=ludisk)
c
c----------------------------------------------------------------------
c   Normalize basis projections to the largest value
c----------------------------------------------------------------------
c
      bmax=basis(2,1)
      do 10 i=2,ndim
        if (bmax.lt.basis(2,i)) bmax=basis(2,i)
 10   continue
c
      do 20 i=1,ndim
        basis(2,i)=basis(2,i)/bmax
 20   continue
      nbss=ndim
c
c----------------------------------------------------------------------
c     Build the full list of basis indices corresponding tothe MTS
c----------------------------------------------------------------------
c
      call lbasix( ixname,namlth,1 )
c
      j=0
c
c----------------------------------------------------------------------
c     Prune the basis set using the new tolerance
c----------------------------------------------------------------------
c
      do 30 i=1,ndim
         if (basis(2,i).ge.btol) then
            j=j+1
            l1(j)=l1(i)
            m1(j)=m1(i)
            k1(j)=k1(i)
            j1(j)=j1(i)
            n1(j)=n1(i)
            i1(j)=i1(i)
            pi1(j)=pi1(i)
            im1(j)=im1(i)
            qi1(j)=qi1(i)
            ps1(j)=ps1(i)
            qs1(j)=qs1(i)
         end if
 30   continue
c 
      write (luttyo,5020) ndim,j,btol
c
c----------------------------------------------------------------------
c   Find MTS corresponding to pruned basis set
c----------------------------------------------------------------------
c
       ndim=j
       nmts=-1
       call mtss( basis,ndim,zero,maxle,minle,maxlo,minlo,maxm,minm,
     #      maxk,mink,maxj,minj,maxn,minn,maxi,mini,maxpi,minpi,
     #      nbas,nmts )
c
       write(luttyo,4200)
       write(luttyo,4210) btol,minle,maxle,minlo,maxlo,minm,maxm,
     #                    mink,maxk, minj,maxj,minn,maxn,
     #                    mini,maxi,minpi,maxpi,nmts,nbas
       write(luttyo,4220)
c
        write(lulog,4200)
        write(lulog,4210) btol,minle,maxle,minlo,maxlo,minm,maxm,
     #                    mink,maxk, minj,maxj,minn,maxn,
     #                    mini,maxi,minpi,maxpi,nmts,nbas
        write(lulog,4220)
c
c---------------------------------------------------------------------
c     Output updated parameter file and basis index file
c---------------------------------------------------------------------
c
c...... no update of truncation values ..........
c
        lemx=maxle
        lomx=maxlo
        mmx=maxm
        kmx=maxk
        jmx=maxj
        nmx=max(iabs(maxn),iabs(minn))
        imx=maxi
        ipnmx=maxpi
c
c----------------------------------------------------------------------     
c   Write out basis set indices for pruned set
c----------------------------------------------------------------------
c
      ndim=j
      write (luttyo,5010) ixname(:namlth)
      open (unit=ludisk,file=ixname(:namlth),status='unknown',
     #     access='sequential',form='unformatted')
      write (ludisk) ndim,lemx,lomx,mmx,kmx,jmx,nmx,imx,ipnmx
      write (ludisk) (l1(i),m1(i),k1(i),j1(i),n1(i),i1(i),
     #               pi1(i),im1(i),qi1(i),ps1(i),qs1(i),i=1,ndim)
      close (unit=ludisk)
      go to 1
c
c----------------------------------------------------------------------
c     exit from program
c----------------------------------------------------------------------
c
 9999 write(luttyo,1000)
      stop
c
c======================================================================
c    format statements
c======================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(28x,'Program : PRUNEL',/,28x,'--------------',/)
c
 1100 format(/,20x,'*** Cannot find file ',a,' ****',/)
 1200 format(/,10x,'*** Error reading file ',a,' ***',/)
 1201 format(/,10x,'*** ''',a,''' is not an EPRLL Version 1.4 file',
     #' ***')
 4200 format(/,2x,76('-')/
     #       3x,'error',7x,'LE',6x,'LO',6x,'M',7x,'K',7x,
     #                     'J',7x,'N',7x,'I',7x,'pI',
     #       4x,'MTS',2x,'pruned'/,12x,8('min max '),' dim',3x,'dim'/)
 4210 format(g10.3,16(i4),2i6)
 4220 format(2x,76('-'))
 5000 format(15x,38('='),/,
     #     15x,'Pruning procedure ',i2,' using file ',a,/,
     #     15x,38('='),/)
 5010 format(/,20x,'*** writing file ',a,' ***')
 5020 format(20x,'MTS basis dimension: ',i7/
     #       20x,'pruned dimension:    ',i7/
     #       20x,'pruning tolerance :  ',f8.4/)
c
c======================================================================
c
      end
